#!/usr/bin/perl -w
##############################################################################
## File		:	signatures.pl
## Version	:	1.0
## Author	:	Nelson Brito<nelson@secunet.com.br>
##
## A ideia deste script e' criar uma base de dados com as digitais
## eletronicas dos arquivo binarios atraves de uma checagem de md5
## checksum.
##
## Apartir deste ponto, ele fara' uma checagem se houve alguma mudanca
## nestas digitais(fingerprint) e informara' ao administrador do sistema
## as mudancas.
##
## Isto e' uma forma de se evitar que sejam instalados rootkits em sua
## maquina, ou verificar instalacoes indesejadas por parte de outro(sik) 
## administrador que compartilha as responsabilidades com voce.
##
## Alem de checagem de digitais, ele ainda checa se foi retirado algum
## arquivo ou incluido.
##
## Espero que curtam o codigo e que seja proveitoso para todos.
##
## Espero conseguir tempo para melhorar ainda mais este script.
##
## Voce pode repassar e/ou modificar este codigo, contanto que deixe claro
## o copyright original do autor e ponha a referencia ao codigo, isto e', 
## nao se esqueca dos creditos.
##
## Caso voce modifique o codigo, mande-me uma copia para que possa
## analizar seu 'hack'.
##
## Para comecar a utilizar o script faca o seguinte:
## 1 - edite suas preferencias no codigo;
## 2 - execute-o direcionando o armazenamento para fingerprint.db:
##     $ ./signatures.pl -o fingerprint.db
## 3 - acrescente uma linha em seu crontab(crontab -e):
##     0 0 * * * /caminho/para/o/signatures.pl
##
## Sugiro que faca um backup do seu primeiro fingerprint.db.
##
## Divirtam-se criancas!
##############################################################################
use strict; 						## confie em mim... =)
use Net::SMTP;
use Getopt::Std;
use MIME::Entity;
use Time::localtime;
use Shell qw(md5sum);

my %opts;
getopts('o:h', \%opts);

## cseg's hack *8@)
$0=~ s#.*/##;

## coloque aqui o diretorio base do signatures.pl
my $signatures_home	=			  "/root/perl/signatures/";

## nao mexa aqui se voce realmente nao sabe o que esta fazendo
my $lt			= 				       localtime();
my $archive		=    $lt->mday."-".$lt->mon."-".($lt->year + 1900); 
my $good_stuff		=			  changed('3caet8jiv3.hm');
my $output_file		=	$opts{'o'} ? $opts{'o'} : $archive . ".db";
my $version		=					     "1.0";
my $counter 		=						 0;

## coloque aqui suas preferencias
my $subject		=      "Alguma coisa de errada com o seu sistema!";
my $smtp_server		=			       "localhost";
my $admin_name		=				   "Administrador";
my $admin_mail		=			      "root\@localhost";
my $script_name		=				   "Signatures.PL";
my $script_mail		=			  "postmaster\@localhost";

## arquivos utilizados pelo script
my $fingerprint_db	=	       $signatures_home . "fingerprint.db";
my $adv_file		=			       "/tmp/advisory.tmp";
my $temp_file		=			     "/tmp/signatures.tmp";

## voce podera' acrescentar quantos mais quiser, sem ter mexer em outra
## parte do codigo
my @locations 		= ("/bin" , 
			   "/sbin" , 
			   "/usr/bin" , 
			   "/usr/sbin" ,
			   "/usr/local/bin" , 
			   "/usr/local/sbin" , 
			   "/usr/X11R6/bin" ,
			   "/opt/bin" ,
			   "/rbin" ,
			   "/rbin/bin" ,
			   "/lib" ,
			   "/usr/lib" ,
			   "/usr/local/lib");

## alguns testes padroes de meus scripts perl
($0 eq $good_stuff) or die "precisa de mais de um neuronio para executar-me\n";
($<==0) or die "precisa de privilegios de root para executar-me\n";

## so' mostrara' o usage se voce especificar na linha de comando
usage($0, $version) if ($opts{'h'});

## background mode, nao e' necessario se voce utilizar com seu crontab
exit(0) if (fork());

## hora da diversao
check_sign($signatures_home . $output_file);

## ja' vi isto em algum lugar, onde sera'? 
sub changed{
	$_ = "$_[0]";
	y/a-z0-9/gvibn9wprud2lmx8z3fa4eq15oy06sjc7kth/;
	y/\2/./;
	return $_;
}

sub usage{
	die "Signatures.PL v. $_[1] - By Nelson Brito<nelson\@secunet.com.br>\n",
	    "Usage: $_[0] (options)\n",
	    "\t-o <file.db>\tdefine o arquivo de saida\t(def:$output_file)\n",
	    "\t-h\t\tmostra como usar o script\n";
}

## pega os md5sum e armazena no arquivo definido em $opts{'o'}
sub check_sign{
	open(FILE, ">" . $_[0]) or die "open($_[0]): $!\n";

	while($counter < @locations){
		if(-d $locations[$counter]){
			my $sub_counter = 0;

			opendir(DIR, $locations[$counter]) or die "open($locations[$counter]): $!\n";
			my @files = readdir(DIR);
			closedir(DIR);

			while($sub_counter < @files){
				if(($files[$sub_counter] eq ".") or ($files[$sub_counter] eq "..")){ }
				else{
					my $single_file = $locations[$counter] . "/" . $files[$sub_counter];
					if((-f $single_file) and (-x $single_file)){
						my $temp_print = md5sum($single_file);
						print FILE $temp_print;
					}
				}
				$sub_counter++;
			}
		}
		$counter++;
	}
	close(FILE);

	build_message($fingerprint_db, $_[0]);
}

## cria mensagem caso algo esteja errado com o fingerprint.db e o atual
sub build_message{
	($_[0] ne $_[1]) or exit(1);

	my $c = 1;
	my $i = 1;
	my $first = 1;

	open(ADVISORY, ">" . $adv_file) or die "opn($adv_file): $!\n";

	open(ONE, $_[0]) or die "open($_[0]): $!\n";
	open(TWO, $_[1]) or die "open($_[1]): $!\n";
	until((eof(ONE)) and (eof(TWO))){
		my $one = <ONE>; chomp($one);
		my $two = <TWO>; chomp($two);
	
		my ($hashone, $fileone) = split(/\s+/, $one);
		my ($hashtwo, $filetwo) = split(/\s+/, $two);
	
			
		if((eof(ONE)) and not (eof(TWO))){
			print ADVISORY "Arquivo $_[0] terminou antes do arquivo\n",
				      "$_[1] na linha $c.\n\n",
				      "Erro de retorno: end of file!\n";
			print ADVISORY "-------------------------------------------------------------------------\n\n",
				       "Copyright(c) 2000 - Nelson Brito<nelson\@secunet.com.br>\n",
					"Security Network AG / IBQN - The Trust Company\n\n",
					"Conheca mais sobre o autor em: http://stderr.sekure.org/\n";
			close(ADVISORY); 

			alert_admin($adv_file);

			exit(1);
		}elsif((eof(TWO)) and not (eof(ONE))){
			print ADVISORY "Arquivo $_[1] terminou antes do arquivo\n",
				      "$_[0] na linha $c.\n",
				      "Erro de retorno: end of file!\n\n";
			print ADVISORY "-------------------------------------------------------------------------\n\n",
				       "Copyright(c) 2000 - Nelson Brito<nelson\@secunet.com.br>\n",
					"Security Network AG / IBQN - The Trust Company\n\n",
					"Conheca mais sobre o autor em: http://stderr.sekure.org/\n";
			close(ADVISORY); 

			alert_admin($adv_file);

			exit(1);
		}

		if($fileone eq $filetwo){
			if($hashone ne $hashtwo){
				print ADVISORY "Mensagem automatica enviada pelo $0, nao responda!\n\n",
					"-------------------------------------------------------------------------\n" if ($i == 1);
				print ADVISORY "Arquivo $fileone teve seu md5sum alterado:\n",
					      "de $hashone para $hashtwo\n\n";
				$i++;
			}
		}
		elsif($fileone ne $filetwo){
			print ADVISORY "Mensagem automatica enviada pelo $0, nao responda!\n\n",
				"-------------------------------------------------------------------------\n" if ($i == 1);
			print ADVISORY "Arquivos $fileone e $filetwo sao diferentes,\n",
				      "porem estao na mesma linha de pesquisa inicial($c).\n\n" if ($first);

			if(not(-f $fileone)){
				print ADVISORY "Arquivo $fileone foi removido de seu sistema.\n\n" if ($first);
			}elsif(not(-f $filetwo)){
				print ADVISORY "Arquivo $filetwo foi removido de seu sistema.\n\n" if ($first);
			}else{
				print ADVISORY "Nao pude determinar se foi excluido,\n",
						"portanto algum arquivo pode ter sido incluido em seu sistema.\n\n" if ($first);
			}

			print ADVISORY "Nao havera' mais checagem apartir desta linha, apague\n",
				      "seu $_[0] e execute novamente o script:\n",
				      "\$ $0 -o $_[0]\n\n",
				      "PS: Verifique o mais rapido possivel seu sistema.\n\n" if ($first);
			$first = 0;
			#i++;
		}	
		$c++;
	}


	print ADVISORY "-------------------------------------------------------------------------\n\n",
		       "Copyright(c) 2000 - Nelson Brito<nelson\@secunet.com.br>\n",
			"Security Network AG / IBQN - The Trust Company\n\n",
			"Conheca mais sobre o autor em: http://stderr.sekure.org/\n" if ($i > 1);
	close(ONE);
	close(TWO);

	close(ADVISORY);
	alert_admin($adv_file) if ($i > 1);
}

## envia mensagem de alerta para o administrador do sistema
sub alert_admin{
	my $message = build MIME::Entity
		Path		=>	"$_[0]",
		Encoding	=>	"quoted-printable",
		Sender		=>	"$admin_name",
		From		=>	"$script_mail($script_name)",
		To		=>	"$admin_mail($admin_name)",
		Subject		=>	"$subject";



	open(MAIL, ">" . $temp_file) or die "open($temp_file): $!\n";
	$message->print(\*MAIL);
	close(MAIL);

	open(MAIL, $temp_file) or die "open($temp_file): $!\n";
	my @mail = <MAIL>;
	close(MAIL);

	my $connection	=	Net::SMTP->new($smtp_server);
	$connection->mail($script_mail);
	$connection->to($admin_mail);
	$connection->data();
	$connection->datasend(@mail);
	$connection->quit;

	unlink($adv_file);
	unlink($temp_file);
}
## fim
